#!/usr/bin/env python3

"""
#   Copyright (C) 2009-2014 Ludovic Rousseau (ludovic.rousseau@free.fr)
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA.
"""


import getinfo

import PyKCS11

if __name__ == "__main__":
    import getopt
    import sys

    def usage():
        """usage"""
        print("Usage:", sys.argv[0], end=" ")
        print("[-p pin][--pin=pin] (use 'NULL' for pinpad)", end=" ")
        print("[-c lib][--lib=lib]", end=" ")
        print("[-h][--help]", end=" ")
        print("[-o][--opensession]")
        print("[-f][--full]")

    try:
        opts, args = getopt.getopt(
            sys.argv[1:],
            "p:c:homf",
            ["pin=", "lib=", "help", "opensession", "mechanisms", "full"],
        )
    except getopt.GetoptError:
        # print help information and exit:
        usage()
        sys.exit(2)

    lib = None
    pin = ""
    open_session = False
    pin_available = False
    list_mechanisms = False
    full = False
    for o, a in opts:
        if o in ("-h", "--help"):
            usage()
            sys.exit()
        if o in ("-p", "--pin"):
            pin = a
            if pin == "NULL":
                pin = None
            pin_available = True
            open_session = True
        if o in ("-c", "--lib"):
            lib = a
        if o in ("-o", "--opensession"):
            open_session = True
        if o in ("-m", "--mechanisms"):
            list_mechanisms = True
        if o in ("-f", "--full"):
            full = True

    gi = getinfo.getInfo(lib)
    gi.getInfo()

    slots = gi.pkcs11.getSlotList()
    print("Available Slots:", len(slots), slots)

    if len(slots) == 0:
        sys.exit(2)

    while True:
        print("waiting...")
        slot = gi.pkcs11.waitForSlotEvent()

        try:
            print(f"Slot {slot} changed")
            gi.getSlotInfo(slot, 0, len(slots))
            if full:
                gi.getSessionInfo(slot, pin)
                gi.getTokenInfo(slot)
                if list_mechanisms:
                    gi.getMechanismInfo(slot)
        except PyKCS11.PyKCS11Error as e:
            print("Error:", e)



# pylint: disable=missing-module-docstring
# pylint: disable=missing-class-docstring
# pylint: disable=missing-function-docstring

import unittest

import PyKCS11


class Testutil(unittest.TestCase):
    def test_empty(self):
        e = PyKCS11.PyKCS11Error(0)
        self.assertEqual(e.value, 0)

    def test_CKR_OK(self):
        e = PyKCS11.PyKCS11Error(PyKCS11.CKR_OK)
        self.assertEqual(e.value, 0)
        self.assertEqual(str(e), "CKR_OK (0x00000000)")

    def test_CKR_PIN_INVALID(self):
        e = PyKCS11.PyKCS11Error(PyKCS11.CKR_PIN_INVALID)
        self.assertEqual(e.value, 0xA1)
        self.assertEqual(str(e), "CKR_PIN_INVALID (0x000000A1)")

    def test_Load(self):
        e = PyKCS11.PyKCS11Error(-1, "Pouet")
        self.assertEqual(e.value, -1)
        self.assertEqual(str(e), "Load (Pouet)")

    def test_raise(self):
        with self.assertRaises(PyKCS11.PyKCS11Error):
            raise PyKCS11.PyKCS11Error(0)

    def test_unknown(self):
        e = PyKCS11.PyKCS11Error(PyKCS11.CKR_VENDOR_DEFINED - 1)
        self.assertEqual(str(e), "Unknown error (0x7FFFFFFF)")

    def test_vendor0(self):
        e = PyKCS11.PyKCS11Error(PyKCS11.CKR_VENDOR_DEFINED, "Pouet")
        self.assertEqual(e.value, PyKCS11.CKR_VENDOR_DEFINED)
        self.assertEqual(str(e), "Vendor error (0x00000000)")

    def test_vendor10(self):
        e = PyKCS11.PyKCS11Error(PyKCS11.CKR_VENDOR_DEFINED + 10)
        self.assertEqual(e.value, PyKCS11.CKR_VENDOR_DEFINED + 10)
        self.assertEqual(str(e), "Vendor error (0x0000000A)")



#! /usr/bin/env python

# execute using:
# python test/test_CK.py

# pylint: disable=missing-module-docstring
# pylint: disable=missing-class-docstring
# pylint: disable=missing-function-docstring

import unittest

import PyKCS11


class TestUtil(unittest.TestCase):
    def test_CKM(self):
        self.assertEqual(PyKCS11.CKM_RSA_PKCS_KEY_PAIR_GEN, 0x00000000)
        self.assertEqual(
            PyKCS11.CKM[PyKCS11.CKM_RSA_PKCS_KEY_PAIR_GEN], "CKM_RSA_PKCS_KEY_PAIR_GEN"
        )

        self.assertEqual(PyKCS11.CKM_VENDOR_DEFINED, 0x80000000)

    def test_CKR(self):
        self.assertEqual(PyKCS11.CKR_VENDOR_DEFINED, 0x80000000)

    def test_CKH(self):
        self.assertEqual(PyKCS11.CKH_USER_INTERFACE, 3)
        self.assertEqual(PyKCS11.CKH["CKH_USER_INTERFACE"], 3)


if __name__ == "__main__":
    unittest.main()



# pylint: disable=missing-module-docstring
# pylint: disable=missing-class-docstring
# pylint: disable=missing-function-docstring

import unittest

from PyKCS11 import PyKCS11


class TestUtil(unittest.TestCase):
    def setUp(self):
        self.pkcs11 = PyKCS11.PyKCS11Lib()
        self.pkcs11.load()

        # get SoftHSM major version
        info = self.pkcs11.getInfo()
        self.SoftHSMversion = info.libraryVersion[0]
        self.manufacturer = info.manufacturerID

        self.slot = self.pkcs11.getSlotList(tokenPresent=True)[0]
        self.session = self.pkcs11.openSession(
            self.slot, PyKCS11.CKF_SERIAL_SESSION | PyKCS11.CKF_RW_SESSION
        )
        self.session.login("1234")

        keyID = (0x22,)
        pubTemplate = [
            (PyKCS11.CKA_CLASS, PyKCS11.CKO_PUBLIC_KEY),
            (PyKCS11.CKA_TOKEN, PyKCS11.CK_TRUE),
            (PyKCS11.CKA_PRIVATE, PyKCS11.CK_FALSE),
            (PyKCS11.CKA_MODULUS_BITS, 0x0400),
            (PyKCS11.CKA_PUBLIC_EXPONENT, (0x01, 0x00, 0x01)),
            (PyKCS11.CKA_ENCRYPT, PyKCS11.CK_TRUE),
            (PyKCS11.CKA_VERIFY, PyKCS11.CK_TRUE),
            (PyKCS11.CKA_VERIFY_RECOVER, PyKCS11.CK_TRUE),
            (PyKCS11.CKA_WRAP, PyKCS11.CK_TRUE),
            (PyKCS11.CKA_LABEL, "My Public Key"),
            (PyKCS11.CKA_ID, keyID),
        ]

        privTemplate = [
            (PyKCS11.CKA_CLASS, PyKCS11.CKO_PRIVATE_KEY),
            (PyKCS11.CKA_TOKEN, PyKCS11.CK_TRUE),
            (PyKCS11.CKA_PRIVATE, PyKCS11.CK_TRUE),
            (PyKCS11.CKA_DECRYPT, PyKCS11.CK_TRUE),
            (PyKCS11.CKA_SIGN, PyKCS11.CK_TRUE),
            (PyKCS11.CKA_SIGN_RECOVER, PyKCS11.CK_TRUE),
            (PyKCS11.CKA_UNWRAP, PyKCS11.CK_TRUE),
            (PyKCS11.CKA_ID, keyID),
        ]

        (self.pubKey, self.privKey) = self.session.generateKeyPair(
            pubTemplate, privTemplate
        )
        self.assertIsNotNone(self.pubKey)
        self.assertIsNotNone(self.privKey)

    def tearDown(self):
        self.session.destroyObject(self.pubKey)
        self.session.destroyObject(self.privKey)

        self.session.logout()
        self.pkcs11.closeAllSessions(self.slot)
        self.pkcs11.unload()
        del self.pkcs11

    def test_sign_integer(self):
        toSign = 1234567890
        mecha = PyKCS11.Mechanism(PyKCS11.CKM_SHA1_RSA_PKCS, None)

        # sign/verify
        try:
            self.session.sign(self.privKey, toSign, mecha)
        except PyKCS11.PyKCS11Error as e:
            self.assertEqual(e.value, -3)

    def test_sign_PKCS(self):
        toSign = "Hello world"
        mecha = PyKCS11.Mechanism(PyKCS11.CKM_SHA1_RSA_PKCS, None)

        # sign/verify
        signature = self.session.sign(self.privKey, toSign, mecha)

        result = self.session.verify(self.pubKey, toSign, signature, mecha)

        self.assertTrue(result)

    def test_sign_PKCS_SHA256(self):
        toSign = "Hello world"
        mecha = PyKCS11.Mechanism(PyKCS11.CKM_SHA256_RSA_PKCS, None)

        # sign/verify
        signature = self.session.sign(self.privKey, toSign, mecha)

        result = self.session.verify(self.pubKey, toSign, signature, mecha)

        self.assertTrue(result)

    def test_sign_X509(self):
        toSign = "Hello world"
        mecha = PyKCS11.Mechanism(PyKCS11.CKM_RSA_X_509, None)

        if self.SoftHSMversion < 2:
            self.skipTest("RSA X.509 only supported by SoftHSM >= 2")

        # sign/verify
        signature = self.session.sign(self.privKey, toSign, mecha)

        result = self.session.verify(self.pubKey, toSign, signature, mecha)

        self.assertTrue(result)

    def test_encrypt_PKCS(self):
        # encrypt/decrypt using CMK_RSA_PKCS (default)
        dataIn = "Hello world"
        encrypted = self.session.encrypt(self.pubKey, dataIn)
        decrypted = self.session.decrypt(self.privKey, encrypted)

        # convert in a string
        text = "".join(map(chr, decrypted))

        self.assertEqual(dataIn, text)

    def test_encrypt_X509(self):
        if self.SoftHSMversion < 2:
            self.skipTest("RSA X.509 only supported by SoftHSM >= 2")

        # encrypt/decrypt using CKM_RSA_X_509
        dataIn = "Hello world!"
        mecha = PyKCS11.Mechanism(PyKCS11.CKM_RSA_X_509, None)
        encrypted = self.session.encrypt(self.pubKey, dataIn, mecha=mecha)
        decrypted = self.session.decrypt(self.privKey, encrypted, mecha=mecha)

        # remove padding NUL bytes
        padding_length = 0
        for e in decrypted:
            if e != 0:
                break
            padding_length += 1
        decrypted = list(decrypted)[padding_length:]

        # convert in a string
        text = "".join(map(chr, decrypted))

        self.assertEqual(dataIn, text)

    def test_RSA_OAEP(self):
        if self.SoftHSMversion < 2:
            self.skipTest("RSA OAEP only supported by SoftHSM >= 2")

        # RSA OAEP
        plainText = "A test string"

        mech = PyKCS11.RSAOAEPMechanism(PyKCS11.CKM_SHA_1, PyKCS11.CKG_MGF1_SHA1)
        cipherText = self.session.encrypt(self.pubKey, plainText, mech)
        decrypted = self.session.decrypt(self.privKey, cipherText, mech)

        text = "".join(map(chr, decrypted))

        self.assertEqual(text, plainText)

    def test_RSA_OAEPwithAAD(self):
        # AAD is "Additional Authentication Data"
        # (pSourceData of CK_RSA_PKCS_OAEP_PARAMS struct)
        if self.SoftHSMversion < 2:
            self.skipTest("RSA OAEP only supported by SoftHSM >= 2")

        if self.manufacturer.startswith("SoftHSM"):
            # SoftHSM indicates in syslog:
            #  "SoftHSM.cpp(12412): pSourceData must be NULL"
            # and returns CKR_ARGUMENTS_BAD
            self.skipTest("'AAD' not (yet) supported.")

        plainText = "A test string"

        # RSA OAEP
        aad = b"sample aad"
        mech = PyKCS11.RSAOAEPMechanism(PyKCS11.CKM_SHA_1, PyKCS11.CKG_MGF1_SHA1, aad)
        cipherText = self.session.encrypt(self.pubKey, plainText, mech)
        decrypted = self.session.decrypt(self.privKey, cipherText, mech)

        text = bytes(decrypted).decode("utf-8")

        self.assertEqual(text, plainText)

    def test_RSA_PSS_SHA1(self):
        if self.SoftHSMversion < 2:
            self.skipTest("RSA PSS only supported by SoftHSM >= 2")

        # RSA PSS
        toSign = "test_RSA_sign_PSS SHA1"

        mech = PyKCS11.RSA_PSS_Mechanism(
            PyKCS11.CKM_SHA1_RSA_PKCS_PSS,
            PyKCS11.CKM_SHA_1,
            PyKCS11.CKG_MGF1_SHA1,
            20,  # size of SHA1 result
        )
        signature = self.session.sign(self.privKey, toSign, mech)
        result = self.session.verify(self.pubKey, toSign, signature, mech)

        self.assertTrue(result)

    def test_RSA_PSS_SHA256(self):
        if self.SoftHSMversion < 2:
            self.skipTest("RSA PSS only supported by SoftHSM >= 2")

        # RSA PSS
        toSign = "test_RSA_sign_PSS SHA256"

        mech = PyKCS11.RSA_PSS_Mechanism(
            PyKCS11.CKM_SHA256_RSA_PKCS_PSS,
            PyKCS11.CKM_SHA256,
            PyKCS11.CKG_MGF1_SHA256,
            32,  # size of SHA256 result
        )
        signature = self.session.sign(self.privKey, toSign, mech)
        result = self.session.verify(self.pubKey, toSign, signature, mech)

        self.assertTrue(result)

    def test_pubKey(self):
        # test CK_OBJECT_HANDLE.__repr__()
        text = str(self.pubKey)
        self.assertIsNotNone(text)



#! /usr/bin/env python

# pylint: disable=missing-module-docstring
# pylint: disable=missing-class-docstring
# pylint: disable=missing-function-docstring

import unittest

from PyKCS11 import PyKCS11

# SHA1 of "abc"
SHA1_abc = [
    0xA9,
    0x99,
    0x3E,
    0x36,
    0x47,
    0x6,
    0x81,
    0x6A,
    0xBA,
    0x3E,
    0x25,
    0x71,
    0x78,
    0x50,
    0xC2,
    0x6C,
    0x9C,
    0xD0,
    0xD8,
    0x9D,
]


class TestUtil(unittest.TestCase):
    def setUp(self):
        self.pkcs11 = PyKCS11.PyKCS11Lib()
        self.pkcs11.load()
        self.slot = self.pkcs11.getSlotList(tokenPresent=True)[0]
        self.session = self.pkcs11.openSession(self.slot, PyKCS11.CKF_SERIAL_SESSION)

    def tearDown(self):
        self.pkcs11.closeAllSessions(self.slot)
        del self.pkcs11

    def test_digest(self):
        digest = self.session.digest("abc")
        self.assertSequenceEqual(digest, SHA1_abc)

    def test_digestSession(self):
        digestSession = self.session.digestSession()
        digestSession.update("abc")
        digest = digestSession.final()
        self.assertSequenceEqual(digest, SHA1_abc)


if __name__ == "__main__":
    unittest.main()
